<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Version details
 *
 * Configurable Reports - A Moodle block for creating customizable reports
 *
 * @package    local_faat
 * @copyright  Daniel Neis Araujo <danielneis@gmail.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace local_faat;

defined('MOODLE_INTERNAL') || die();

require_once($CFG->libdir . '/externallib.php');
require_once($CFG->libdir . '/gradelib.php');
require_once($CFG->libdir . '/grade/grade_item.php');
require_once($CFG->dirroot . '/course/lib.php');
require_once($CFG->dirroot . '/course/modlib.php');
require_once($CFG->dirroot . '/mod/quiz/mod_form.php');
require_once($CFG->dirroot . '/question/editlib.php');
require_once($CFG->dirroot . '/question/engine/lib.php');
require_once($CFG->dirroot . '/mod/quiz/locallib.php');
require_once($CFG->dirroot . '/mod/quiz/accessmanager.php');
require_once($CFG->dirroot . '/mod/quiz/attemptlib.php');

use external_api;
use external_function_parameters;
use external_value;
use external_single_structure;
use external_multiple_structure;
use external_warnings;
use context_course;
use context_module;
use context_system;
use question_bank;
use question_edit_contexts;
use quiz;
use quiz_attempt;
use moodle_quiz_exception;

/**
 * This is the external API for this component.
 *
 * @copyright  Daniel Neis Araujo <danielneis@gmail.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class external extends external_api
{

    /**
     * create_quiz parameters.
     *
     * @return external_function_parameters
     */
    public static function create_quiz_parameters()
    {
        return new external_function_parameters(
            array(
                'courseid' => new external_value(PARAM_INT, 'The course id', VALUE_REQUIRED),
                'quiz' => new external_value(PARAM_RAW, 'The JSON with all data to create quiz', VALUE_REQUIRED),
            )
        );
    }

    /**
     * Returns data of given report id.
     *
     * @param int $courseid
     * @param string $data The quiz JSON
     * @return array An array with a 'data' JSON string and a 'warnings' string
     */
    public static function create_quiz($courseid, $quiz)
    {
        global $CFG, $DB;
        $params = self::validate_parameters(self::create_quiz_parameters(), ['courseid' => $courseid, 'quiz' => $quiz]);

        if ($params['courseid'] == SITEID) {
            $context = context_system::instance();
        } else {
            $context = context_course::instance($params['courseid']);
        }

        $params['quiz'] = json_decode(stripslashes($quiz));
        self::validate_context($context);

        $course = $DB->get_record('course', ['id' => $params['courseid']]);
        list($module, $context, $cw, $cm, $data) = prepare_new_moduleinfo_data($course, 'quiz', $params['quiz']->section);

        $mformclassname = 'mod_quiz_mod_form';
        $mform = new $mformclassname($data, $cw->section, $cm, $course);
        $mform->set_data($data);
        $fromform = fullclone($data);
        $fromform->name = $params['quiz']->name;
        $fromform->introeditor = [
            "text" => base64_decode(str_replace(" ","+",$params['quiz']->intro)),
            // "text" => $params['quiz']->intro,
            "format" => "1",
        ];
        $fromform->section = $params['quiz']->section;
        $fromform->attempts = $params['quiz']->attempts;
        $fromform->overduehandling = $params['quiz']->overduehandling;
        $fromform->cmidnumber = $params['quiz']->cmidnumber;
        $fromform->attempts = 1;
        $fromform->questionsperpage = 1;
        $fromform->quizpassword = '';
        $fromform->preferredbehaviour = 'deferredfeedback';
        if ($params['quiz']->timeopen) {
            $fromform->timeopen = $params['quiz']->timeopen;
        }
        if ($params['quiz']->timeclose) {
            $fromform->timeclose = $params['quiz']->timeclose;
        }
        if ($params['quiz']->shuffleanswers) {
            $fromform->shuffleanswers = $params['quiz']->shuffleanswers;
        }
        if ($params['quiz']->timelimit) {
            $fromform->timelimit = $params['quiz']->timelimit;
        }
        /*
            "timeclose" => $params['quiz']->timeclose,
            [
                "day" => "18",
                "month" => "7",
                "year" => "2021",
                "hour" => "18",
                "minute" => "40",
                "enabled" => "1"
            ],
            "timelimit" => $params['quiz'] -> timelimit,
            [
                "number" => "150",
                "timeunit" => "60",
                "enabled" => "1"
            ],
            'shuffleanswers' => 1,

            'navmethod' => "free",
            */
        $fromform = add_moduleinfo($fromform, $course, $mform);

        $cmid = $fromform->coursemodule;

        return $cmid;
    }

    /**
     * create_quiz return
     *
     * @return external_description
     */
    public static function create_quiz_returns()
    {
        return new external_value(PARAM_INT, 'The coure module id of the quiz');
    }

    /**
     * update_quiz parameters.
     *
     * @return external_function_parameters
     */
    public static function update_quiz_parameters()
    {
        return new external_function_parameters(
            array(
                'quiz' => new external_value(PARAM_RAW, 'The JSON with all data to create quiz', VALUE_REQUIRED),
            )
        );
    }

    public static function update_quiz($quiz)
    {
        global $DB;

        $params = self::validate_parameters(self::update_quiz_parameters(), ['quiz' => $quiz]);

        $params['quiz'] = json_decode($params['quiz']);

        // Check the course module exists.
        $cm = get_coursemodule_from_id('', $params['quiz']->cmid, 0, false, MUST_EXIST);

        // Check the course exists.
        $course = $DB->get_record('course', array('id' => $cm->course), '*', MUST_EXIST);

        $quiz = $DB->get_record('quiz', ['id' => $cm->instance]);

        if (!empty($params['quiz']->name)) {
            $quiz->name = $params['quiz']->name;
        }
        if (!empty($params['quiz']->intro)) {
            $quiz->intro = base64_decode($params['quiz']->intro);
        }
        if (!empty($params['quiz']->timeopen)) {
            $quiz->timeopen = $params['quiz']->timeopen;
        }
        if (!empty($params['quiz']->timeclose)) {
            $quiz->timeclose = $params['quiz']->timeclose;
        }
        if (!empty($params['quiz']->timelimit)) {
            $quiz->timelimit = $params['quiz']->timelimit;
        }

        $DB->update_record('quiz', $quiz);

        \course_modinfo::build_course_cache($course);

        return $cm->id;
    }

    /**
     * update_quiz return
     *
     * @return external_description
     */
    public static function update_quiz_returns()
    {
        return new external_value(PARAM_INT, 'The coure module id of the quiz');
    }

    /**
     * delete_quiz parameters.
     *
     * @return external_function_parameters
     */
    public static function delete_quiz_parameters()
    {
        return new external_function_parameters(
            array(
                'cmid' => new external_value(PARAM_INT, 'The course module id of the quiz to be deleted', VALUE_REQUIRED),
            )
        );
    }

    public static function delete_quiz($cmid)
    {
        course_delete_module($cmid, true);
        return '';
    }

    /**
     * delete_quiz return
     *
     * @return external_description
     */
    public static function delete_quiz_returns()
    {
        return new external_value(PARAM_RAW, 'Empty string');
    }

    /**
     * delete_question parameters.
     *
     * @return external_function_parameters
     */
    public static function delete_question_parameters()
    {
        return new external_function_parameters(
            array(
                'cmid' => new external_value(PARAM_INT, 'The course module id of the question to be deleted', VALUE_REQUIRED),
                'questionid' => new external_value(PARAM_INT, 'The id of the question to be deleted from the quiz', VALUE_REQUIRED),
            )
        );
    }

    public static function delete_question($cmid, $questionid)
    {
        global $DB;
        $params = self::validate_parameters(
            self::delete_question_parameters(),
            ['cmid' => $cmid, 'questionid' => $questionid]
        );
        $cm = get_coursemodule_from_id('', $params['cmid'], 0, false, MUST_EXIST);

        $quiz = $DB->get_record('quiz', array('id' => $cm->instance), '*', MUST_EXIST);
        $course = $DB->get_record('course', array('id' => $quiz->course), '*', MUST_EXIST);
        $quizobj = new \quiz($quiz, $cm, $course);
        $structure = $quizobj->get_structure();

        if (!$slot = $DB->get_record('quiz_slots', ['quizid' => $quiz->id, 'questionid' => $params['questionid']])) {
            throw new \moodle_exception('AJAX commands.php: Bad slot ID ' . $id);
        }
        $structure->remove_slot($slot->slot);
        quiz_delete_previews($quiz);
        quiz_update_sumgrades($quiz);
        return '';
    }

    /**
     * delete_question return
     *
     * @return external_description
     */
    public static function delete_question_returns()
    {
        return new external_value(PARAM_RAW, 'Empty string');
    }

    /**
     * create_question parameters.
     *
     * @return external_function_parameters
     */
    public static function create_question_parameters()
    {
        return new external_function_parameters(
            array(
                'cmid' => new external_value(PARAM_INT, 'The course module of the quiz', VALUE_REQUIRED),
                'question' => new external_value(PARAM_RAW, 'The JSON with all data to create question', VALUE_REQUIRED),
            )
        );
    }

    public function create_question($cmid, $question)
    {
        global $USER;

        $params = self::validate_parameters(self::create_question_parameters(), ['cmid' => $cmid, 'question' => $question]);

        $params['question'] = json_decode(stripslashes($params['question']));
        list($module, $cm) = get_module_from_cmid($params['cmid']);
        $thiscontext = context_module::instance($params['cmid']);

        $contexts = new question_edit_contexts($thiscontext);

        $defaultcategory = question_make_default_categories($contexts->all());
        $category = $defaultcategory;

        $question = new \stdClass();
        $question->id = '';
        $question->contextid = $thiscontext->id;
        $question->category = $category->id;
        //       $question->category = "{$category->id},{$category->contextid}";
        $question->qtype = $params['question']->qtype;
        $question->createdby = $USER->id; // TODO: alterar pro ID do usuário do professor, aqui o do sistema.

        $qtypeobj = question_bank::get_qtype($question->qtype);

        // Check permissions
        $question->formoptions = new \stdClass();
        $question->formoptions->canedit = true;
        $question->formoptions->canmove = true;
        $question->formoptions->cansaveasnew = false;
        $question->formoptions->repeatelements = true;

        $mform = $qtypeobj->create_editing_form('question.php', $question, $category, $contexts, true);

        $toform = fullclone($question); // send the question object and a few more parameters to the form

        $mform->set_data($toform);

        $fromform = [
            'cmid' => $params['cmid'],
            'name' => $params['question']->name,
            'courseid' => $params['question']->courseid,
            'category' => $question->category,
            'questiontext' => [
                // "text" => $params['question']->text,
                "text" => base64_decode(str_replace(" ","+", $params['question']->text)),
                "format" => "1",
            ],
            'appendqnumstring' => 'addquestion',
            'submitbutton' => 'Salvar mudanças',
            'defaultmark' => empty($params['question']->peso) ? 1 : $params['question']->peso,
            'savechanges' => true
        ];

        if ($question->qtype == 'multichoice') {
            $fromform["single"] = "1";
            $fromform['showstandardinstruction'] = "1";
            $fromform["shuffleanswers"] = "1";
            $fromform["answernumbering"] = "abc";
          /*  $fromform['answer'] = [
                [
                    "text" => $params['question']->answer->a,
                    "format" => "1",
                ],
                [
                    "text" => $params['question']->answer->b,
                    "format" => "1",
                ],
                [
                    "text" => $params['question']->answer->c,
                    "format" => "1",
                ],
                [
                    "text" => $params['question']->answer->d,
                    "format" => "1",
                ],
                [
                    "text" => $params['question']->answer->e,
                    "format" => "1",
                ]
            ];
            if ($params['question']->alternativa_correta == 'A') {
                $fromform['fraction'] = [
                    "1.0",
                    "0.0",
                    "0.0",
                    "0.0",
                    "0.0"
                ];
            } elseif ($params['question']->alternativa_correta == 'B') {
                $fromform['fraction'] = [
                    "0.0",
                    "1.0",
                    "0.0",
                    "0.0",
                    "0.0"
                ];
            } elseif ($params['question']->alternativa_correta == 'C') {
                $fromform['fraction'] = [
                    "0.0",
                    "0.0",
                    "1.0",
                    "0.0",
                    "0.0"
                ];
            } elseif ($params['question']->alternativa_correta == 'D') {
                $fromform['fraction'] = [
                    "0.0",
                    "0.0",
                    "0.0",
                    "1.0",
                    "0.0"
                ];
            } elseif ($params['question']->alternativa_correta == 'E') {
                $fromform['fraction'] = [
                    "0.0",
                    "0.0",
                    "0.0",
                    "0.0",
                    "1.0"
                ];
            }else{
                $fromform['fraction'] = [
                    "1.0",
                    "0.0",
                    "0.0",
                    "0.0",
                    "0.0"
                ];

            }

            $fromform['feedback'] = [
                ['format' => 1],
                ['format' => 1],
                ['format' => 1],
                ['format' => 1],
                ['format' => 1],
            ];
*/
if (!empty($params['question']->answer->c) && !empty($params['question']->answer->d) && !empty($params['question']->answer->e)) {
                $fromform['answer'] = [
                    [
                        "text" => $params['question']->answer->a,
                        "format" => "1",
                    ],
                    [
                        "text" => $params['question']->answer->b,
                        "format" => "1",
                    ],
                    [
                        "text" => $params['question']->answer->c,
                        "format" => "1",
                    ],
                    [
                        "text" => $params['question']->answer->d,
                        "format" => "1",
                    ],
                    [
                        "text" => $params['question']->answer->e,
                        "format" => "1",
                    ]
                ];
                if ($params['question']->alternativa_correta == 'A') {
                    $fromform['fraction'] = [
                        "1.0",
                        "0.0",
                        "0.0",
                        "0.0",
                        "0.0"
                    ];
                } elseif ($params['question']->alternativa_correta == 'B') {
                    $fromform['fraction'] = [
                        "0.0",
                        "1.0",
                        "0.0",
                        "0.0",
                        "0.0"
                    ];
                } elseif ($params['question']->alternativa_correta == 'C') {
                    $fromform['fraction'] = [
                        "0.0",
                        "0.0",
                        "1.0",
                        "0.0",
                        "0.0"
                    ];
                } elseif ($params['question']->alternativa_correta == 'D') {
                    $fromform['fraction'] = [
                        "0.0",
                        "0.0",
                        "0.0",
                        "1.0",
                        "0.0"
                    ];
                } elseif ($params['question']->alternativa_correta == 'E') {
                    $fromform['fraction'] = [
                        "0.0",
                        "0.0",
                        "0.0",
                        "0.0",
                        "1.0"
                    ];
                }else{
                    $fromform['fraction'] = [
                        "1.0",
                        "0.0",
                        "0.0",
                        "0.0",
                        "0.0"
                    ];
                }

                $fromform['feedback'] = [
                    ['format' => 1],
                    ['format' => 1],
                    ['format' => 1],
                    ['format' => 1],
                    ['format' => 1],
                ];
            } elseif (!empty($params['question']->answer->c) && !empty($params['question']->answer->d)) {
                $fromform['answer'] = [
                    [
                        "text" => $params['question']->answer->a,
                        "format" => "1",
                    ],
                    [
                        "text" => $params['question']->answer->b,
                        "format" => "1",
                    ],
                    [
                        "text" => $params['question']->answer->c,
                        "format" => "1",
                    ],
                    [
                        "text" => $params['question']->answer->d,
                        "format" => "1",
                    ]
                ];
                if ($params['question']->alternativa_correta == 'A') {
                    $fromform['fraction'] = [
                        "1.0",
                        "0.0",
                        "0.0",
                        "0.0"
                    ];
                } elseif ($params['question']->alternativa_correta == 'B') {
                    $fromform['fraction'] = [
                        "0.0",
                        "1.0",
                        "0.0",
                        "0.0"
                    ];
                } elseif ($params['question']->alternativa_correta == 'C') {
                    $fromform['fraction'] = [
                        "0.0",
                        "0.0",
                        "1.0",
                        "0.0"
                    ];
                } elseif ($params['question']->alternativa_correta == 'D') {
                    $fromform['fraction'] = [
                        "0.0",
                        "0.0",
                        "0.0",
                        "1.0"
                    ];
                }else{
                    $fromform['fraction'] = [
                        "1.0",
                        "0.0",
                        "0.0",
                        "0.0",
                    ];
                }

                $fromform['feedback'] = [
                    ['format' => 1],
                    ['format' => 1],
                    ['format' => 1],
                    ['format' => 1],
                ];
            }
            $fromform['correctfeedback'] = '';
            $fromform['partiallycorrectfeedback'] = '';
            $fromform['incorrectfeedback'] = '';
        } else {
            $fromform['responseformat'] = "editor";
            $fromform['responserequired'] = 1;
            $fromform['responsefieldlines'] = 15;
            $fromform['attachments'] = 0;
            $fromform['attachmentsrequired'] = 0;
            $fromform['maxbytes'] = 0;
            $fromform['graderinfo'] = ['format' => 1, 'text' => ''];
            $fromform['responsetemplate'] = ['format' => 1, 'text' => ''];
        }

        // if (empty($question->id)) {
        //     $qtypeobj->save_defaults_for_new_questions((object)$fromform);
        // }
        $question = $qtypeobj->save_question($question, (object)$fromform);

        // Purge this question from the cache.
        question_bank::notify_question_edited($question->id);

        quiz_require_question_use($question->id);
        quiz_add_quiz_question($question->id, $module, 0);
        quiz_delete_previews($module);
        quiz_update_sumgrades($module);

        return $question->id;
    }

    /**
     * create_question return
     *
     * @return external_description
     */
    public static function create_question_returns()
    {
        return new external_value(PARAM_INT, 'The id of the question');
    }

    /**
     * update_question parameters.
     *
     * @return external_function_parameters
     */
    public static function update_question_parameters()
    {
        return new external_function_parameters(
            array(
                'question' => new external_value(PARAM_RAW, 'The JSON with all data to update question', VALUE_REQUIRED),
            )
        );
    }

    public static function update_question($question) {
        global $DB;

        $params = self::validate_parameters(self::update_question_parameters(), ['question' => $question]);

        $params['question'] = json_decode(stripslashes($params['question']));
	sleep(1);	

	$cm = get_coursemodule_from_id('', $params['question']->cmid, 0, false, MUST_EXIST);

        // Check the course exists.
        $course = $DB->get_record('course', array('id' => $cm->course), '*', MUST_EXIST);

        if (!$question = $DB->get_record('question', array('id' => $params['question']->id))) {
            print_error('questiondoesnotexist', 'question', $returnurl);
        }

        // Validate the question category.
        if (!$category = $DB->get_record('question_categories', array('id' => $question->category))) {
            print_error('categorydoesnotexist', 'question', $returnurl);
        }

        if (!empty($params['question']->text)) {
            $question->questiontext = base64_decode(str_replace(" ","+",$params['question']->text));
        }
        if (!empty($params['question']->peso)) {
            $question->defaultmark = $params['question']->peso;
        }
        $DB->update_record('question', $question);

        if ($params['question']->qtype == 'multichoice') {
            $question->single = "1";
            $question->shuffleanswers = "1";
            $question->answernumbering = "abc";

	  if (!empty($params['question']->answer->a)) {
                $sql = "SELECT * FROM {question_answers} WHERE question = ? LIMIT 0,1";
                if ($answer = $DB->get_record_sql($sql, [$question->id])) {
                    $answer->answer = $params['question']->answer->a;
                }
               /* if ($params['question']->alternativa_correta == 'A') {
                    $answer->fraction = "1.0";
                }*/
                $DB->update_record('question_answers', $answer);
            }

	   if (!empty($params['question']->answer->b)) {
                $sql = "SELECT * FROM {question_answers} WHERE question = ? LIMIT 1,1";
                if ($answer = $DB->get_record_sql($sql, [$question->id])) {
                    $answer->answer = $params['question']->answer->b;
                }
               /* if ($params['question']->alternativa_correta == 'A') {
                    $answer->fraction = "1.0";
                }*/
                $DB->update_record('question_answers', $answer);
            }


	   if (!empty($params['question']->answer->c)) {
                $sql = "SELECT * FROM {question_answers} WHERE question = ? LIMIT 2,1";
                if ($answer = $DB->get_record_sql($sql, [$question->id])) {
                    $answer->answer = $params['question']->answer->c;
                }
               /* if ($params['question']->alternativa_correta == 'A') {
                    $answer->fraction = "1.0";
                }*/
                $DB->update_record('question_answers', $answer);
            }


	   if (!empty($params['question']->answer->d)) {
                $sql = "SELECT * FROM {question_answers} WHERE question = ? LIMIT 3,1";
                if ($answer = $DB->get_record_sql($sql, [$question->id])) {
                    $answer->answer = $params['question']->answer->d;
                }
               /* if ($params['question']->alternativa_correta == 'A') {
                    $answer->fraction = "1.0";
                }*/
                $DB->update_record('question_answers', $answer);
            }

	   if (!empty($params['question']->answer->e)) {
                $sql = "SELECT * FROM {question_answers} WHERE question = ? LIMIT 4,1";
                if ($answer = $DB->get_record_sql($sql, [$question->id])) {
                    $answer->answer = $params['question']->answer->e;
                }
               /* if ($params['question']->alternativa_correta == 'A') {
                    $answer->fraction = "1.0";
                }*/
                $DB->update_record('question_answers', $answer);
            }
        }

        // Purge this question from the cache.
        question_bank::notify_question_edited($question->id);

        return $question->id;
    }

    /**
     * update_question return
     *
     * @return external_description
     */
    public static function update_question_returns()
    {
        return new external_value(PARAM_INT, 'The id of the question');
    }

    /**
     * grade_question parameters.
     *
     * @return external_function_parameters
     */

     public static function grade_question_parameters()
    {
        return new external_function_parameters(
            array(
                'attemptid' => new external_value(PARAM_INT, 'The id of the attempt', VALUE_REQUIRED),
                'slot' => new external_value(PARAM_INT, 'The quiz slot', VALUE_REQUIRED),
                'questionattemptid' => new external_value(PARAM_INT, 'The questionattempt id', VALUE_REQUIRED),
                'grade' => new external_value(PARAM_RAW, 'The grade for the question', VALUE_REQUIRED),
                'maxgrade' => new external_value(PARAM_RAW, 'The grade for the question', VALUE_REQUIRED),
                'feedback' => new external_value(PARAM_RAW, 'The feedback for the question', VALUE_REQUIRED),
            )
        );
    }

    public static function grade_question($attemptid, $slot, $questionattemptid, $grade, $maxgrade, $feedback)
    {
        global $DB;
        $params = self::validate_parameters(
            self::grade_question_parameters(),
            [
                'attemptid' => $attemptid,
                'slot' => $slot,
                'questionattemptid' => $questionattemptid,
                'grade' => $grade,
		"maxgrade" => $maxgrade,
                'feedback' => $feedback
                ]
        );
        $attemptobj = quiz_create_attempt_handling_errors($params['attemptid'], null);
        // $attemptobj->preload_all_attempt_step_users();
        $context = context_course::instance($attemptobj->get_course()->id);
        self::validate_context($context);

        $student = $DB->get_record('user', array('id' => $attemptobj->get_userid()));
        // Can only grade finished attempts.
        if (!$attemptobj->is_finished()) {
            return 'attempt not finished';
        }

        if (\question_engine::is_manual_grade_in_range($attemptobj->get_uniqueid(), $slot)) {

            $sql = "SELECT MAX(sequencenumber)+1 as currentsequence FROM {question_attempt_steps} WHERE questionattemptid = ?";
            $sequence = $DB->get_record_sql($sql, [$questionattemptid]);
            $sequence = $sequence->currentsequence;
            $step = [
                'questionattemptid' => $params['questionattemptid'],
                'sequencenumber' => $sequence,
                'state' => 'mangrpartial',
                'fraction' => $params['grade'],
                'timecreated' => time(),
                'userid' => 2 // Substituir pelo ID do usuário do professor.
            ];
         //  var_dump($step); die();
            $stepid = $DB->insert_record('question_attempt_steps', $step);

            $stepdata = [
                'attemptstepid' => $stepid,
                'name' => '-comment',
                'value' => $params['feedback'] // Substituir pelo feedback do professor
            ];
            $DB->insert_record('question_attempt_step_data', $stepdata);

            $stepdata = [
                'attemptstepid' => $stepid,
                'name' => '-commentformat',
                'value' => '1'
            ];
            $DB->insert_record('question_attempt_step_data', $stepdata);

            $stepdata = [
                'attemptstepid' => $stepid,
                'name' => '-mark',
                'value' => ($params['grade'] > $params['maxgrade']) ? $params['maxgrade'] : $params['grade'] // Substituir pela nota do professor.
            ];
            $DB->insert_record('question_attempt_step_data', $stepdata);

            $stepdata = [
                'attemptstepid' => $stepid,
                'name' => '-maxmark',
                'value' => $params['maxgrade'] // Substituir pela nota máxima da questão.
            ];
            $DB->insert_record('question_attempt_step_data', $stepdata);

            $attemptrecord = $DB->get_record('quiz_attempts', ['id' => $params['attemptid']]);
            $quiz = $DB->get_record('quiz', ['id' => $attemptrecord->quiz]);
            $quba = \question_engine::load_questions_usage_by_activity($attemptrecord->uniqueid);

            $attemptrecord->timemodified = time();
            if ($attemptrecord->state == \quiz_attempt::FINISHED) {
                $attemptrecord->sumgrades = $quba->get_total_mark();
            }
            $DB->update_record('quiz_attempts', $attemptrecord);

            if ($attemptrecord->state == \quiz_attempt::FINISHED) {
                quiz_save_best_grade($quiz, $attemptrecord->userid);
            }

            // Log this action.
            $params = array(
                'objectid' => $attemptobj->get_question_attempt($slot)->get_question()->id,
                'courseid' => $attemptobj->get_courseid(),
                'context' => context_module::instance($attemptobj->get_cmid()),
                'other' => array(
                    'quizid' => $attemptobj->get_quizid(),
                    'attemptid' => $attemptobj->get_attemptid(),
                    'slot' => $slot
                )
            );

            $event = \mod_quiz\event\question_manually_graded::create($params);
            $event->trigger();
            return '200';
        }
    }

    /**
     * grade_question return
     *
     * @return external_description
     */
    public static function grade_question_returns() {
        return new external_value(PARAM_TEXT, 'Empty if success, error message otherwise');
    }

    public static function get_quiz_attempts_parameters() {
        return new external_function_parameters(
            array(
                'cmid' => new external_value(PARAM_INT, 'The course module of the quiz', VALUE_REQUIRED),
            )
        );
    }

    public function get_quiz_attempts($cmid) {
        global $USER, $DB;
        $params = self::validate_parameters(self::get_quiz_attempts_parameters(), ['cmid' => $cmid]);

        if (!$cm = get_coursemodule_from_id('quiz', $params['cmid'])) {
            print_error('invalidcoursemodule');
        }

        if (!$course = $DB->get_record('course', array('id' => $cm->course))) {
            print_error('coursemisconf');
        }
        if (!$quiz = $DB->get_record('quiz', array('id' => $cm->instance))) {
            print_error('invalidcoursemodule');
        }

        self::update_review($params['cmid']);
        $report = new \local_faat\quiz_report();
        return $report->display($quiz, $cm, $course);
    }

    public static function get_quiz_attempts_returns() {
        return new external_value(PARAM_RAW, 'json with table data');
    }

    /**
     * update_quiz parameters.
     *
     * @return external_function_parameters
     */
    public static function update_review_parameters()
    {
        return new external_function_parameters(
            array(
                'quizid' => new external_value(PARAM_INT, 'Quiz ID', VALUE_REQUIRED),
            )
        );
    }

    public static function update_review($quizid)
    {
        global $DB;

        $params = self::validate_parameters(self::update_review_parameters(), ['quizid' => $quizid]);

        // Check the course module exists.
        $cm = get_coursemodule_from_id('', $params['quizid'], 0, false, MUST_EXIST);

        // Check the course exists.
        $course = $DB->get_record('course', array('id' => $cm->course), '*', MUST_EXIST);

        $quiz = $DB->get_record('quiz', ['id' => $cm->instance]);

        $quiz->grade = 10;
        $quiz->reviewattempt = 65552;
        $quiz->reviewcorrectness = 16;
        $quiz->reviewmarks = 16;
        $quiz->reviewspecificfeedback = 16;
        $quiz->reviewgeneralfeedback = 16;
        $quiz->reviewrightanswer = 16;
        $quiz->reviewoverallfeedback = 16;

        $DB->update_record('quiz', $quiz);

        \course_modinfo::build_course_cache($course);

        return $cm->id;
    }

    /**
     * update_review return
     *
     * @return external_description
     */
    public static function update_review_returns()
    {
        return new external_value(PARAM_INT, 'The course module id of the quiz');
    }

    public static function quiz_override_parameters() {
        return new external_function_parameters(
            array(
                'userid' => new external_value(PARAM_INT, 'The user id', VALUE_REQUIRED),
                'quiz' => new external_value(PARAM_INT, 'The course module id of the quiz', VALUE_REQUIRED),
                'timeclose' => new external_value(PARAM_INT, 'The unix timestamp with close date', VALUE_REQUIRED),
            )
        );
    }

    public static function quiz_override($userid, $quiz, $timeclose) {
        global $CFG, $DB;
        $params = self::validate_parameters(self::quiz_override_parameters(),
            ['userid' => $userid, 'quiz' => $quiz, 'timeclose' => $timeclose]);

        $cm = $DB->get_record('course_modules', ['id' => $params['quiz']]);
        $fromform = (object)[
          'timeclose' => $params['timeclose'],
          'quiz' => $cm->instance,
          'userid' => $params['userid'],
        ];
        if ($overrideid = $DB->get_field('quiz_overrides', 'id', ['userid' => $params['userid'], 'quiz' => $cm->instance])) {
            $fromform->id = $overrideid;
            $DB->update_record('quiz_overrides', $fromform);
        } else {
            $fromform->id = $DB->insert_record('quiz_overrides', $fromform);
        }
        $cachekey = "{$fromform->quiz}_u_{$fromform->userid}";
        // \cache::make('mod_quiz', 'overrides')->delete($cachekey);
        return $fromform->id;
    }

    public static function quiz_override_returns() {
        return new external_value(PARAM_INT, 'The override id');
    }

    public static function quiz_process_attempt_parameters() {
        return new external_function_parameters(
            array(
                'quiz' => new external_value(PARAM_INT, 'The course module id of the quiz', VALUE_REQUIRED),
                'userid' => new external_value(PARAM_INT, 'The user id', VALUE_REQUIRED),
                'response' => new external_value(PARAM_INT, 'The response', VALUE_REQUIRED),
            )
        );
    }

    public static function quiz_process_attempt($quiz, $userid, $response) {
        global $DB, $USER;

        $params = self::validate_parameters(self::quiz_process_attempt_parameters(),
            ['userid' => $userid, 'quiz' => $quiz, 'response' => $response]);

        // Request and permission validation.
        $cm = $DB->get_record('course_modules', ['id' => $params['quiz']]);
        $quiz = $DB->get_record('quiz', array('id' => $cm->instance), '*', MUST_EXIST);
        list($course, $cm) = get_course_and_cm_from_instance($quiz, 'quiz');
        $context = context_module::instance($cm->id);

        $quizobj = quiz::create($cm->instance, $params['userid']);

        // Create an object to manage all the other (non-roles) access rules.
        $timenow = time();
        $accessmanager = $quizobj->get_access_manager($timenow);

        $forcenew = false;

        $USER = $DB->get_record('user', ['id' => $params['userid']]);

        // Validate permissions for creating a new attempt and start a new preview attempt if required.
        list($currentattemptid, $attemptnumber, $lastattempt, $messages, $page) =
            quiz_validate_new_attempt($quizobj, $accessmanager, $forcenew, -1, false);

        if ($currentattemptid) {
            if ($lastattempt->state == quiz_attempt::OVERDUE) {
                throw new moodle_quiz_exception($quizobj, 'stateoverdue');
            } else {
                throw new moodle_quiz_exception($quizobj, 'attemptstillinprogress');
            }
        }
        if (!empty($messages)) {
            foreach($messages as $message) {
                $warnings[] = array(
                    'item' => 'quiz',
                    'itemid' => $quiz->id,
                    'warningcode' => '1',
                    'message' => clean_text($message, PARAM_TEXT)
                );
            }
            return ['warnings' => $warnings, 'attemptid' => null];
        }

        $offlineattempt = false;
        $attempt = quiz_prepare_and_start_new_attempt($quizobj, $attemptnumber, $lastattempt, $offlineattempt);

        $attemptid = $attempt->id;
        // Add a page, required by validate_attempt.
        $attemptobj = quiz_attempt::create($attemptid);
        $attemptobj->handle_if_time_expired(time(), true);

        $failifoverdue = true;
        // Attempt closed?.
        if ($attemptobj->is_finished()) {
            throw new moodle_quiz_exception($attemptobj->get_quizobj(), 'attemptalreadyclosed');
        } else if ($failifoverdue && $attemptobj->get_state() == quiz_attempt::OVERDUE) {
            throw new moodle_quiz_exception($attemptobj->get_quizobj(), 'stateoverdue');
        }

        $transaction = $DB->start_delegated_transaction();
        // Create the $_POST object required by the question engine.
        $data = [
            [
                'name' => 'q' . $attemptobj->get_uniqueid() . ':1_answer',
                'value' => $params['response']
            ],
            [
                'name' => 'q' . $attemptobj->get_uniqueid() . ':1_:sequencecheck',
                'value' => 1
            ],
            [
                'name' => 'q' . $attemptobj->get_uniqueid() . ':1_:flagged',
                'value' => 0
            ],
        ];
        $_POST = array();
        foreach ($data as $element) {
            $_POST[$element['name']] = $element['value'];
            // Some deep core functions like file_get_submitted_draft_itemid() also requires $_REQUEST to be filled.
            $_REQUEST[$element['name']] = $element['value'];
        }
        $_POST['attempt'] = $attemptobj->get_attemptid();
        $_POST['thispage'] = 0;
        $_POST['nextpage'] = -1;
        $_POST['timeup'] = 0;
        $_POST['slots'] = 1;

        $timenow = time();
        $finishattempt = $timenow;

        $attemptobj->process_auto_save($timenow);
        $transaction->allow_commit();

        $result = $attemptobj->process_attempt($timenow, $finishattempt, false, 0);

        return ['attemptid' => $attemptobj->get_attemptid(), 'warnings' => []];
    }

    public static function quiz_process_attempt_returns() {
        return new external_single_structure([
            'attemptid' => new external_value(PARAM_INT, 'The attempt id'),
            'warnings' => new external_warnings(),
        ]);
    }
}
